#include <Wire.h>
#include "IOpins.h"


//============================================================== Encoder input variables ========================================================================

unsigned long lpulse;                                         // width of left and right encoder pulses in uS
unsigned long rpulse;                                         // width of left and right encoder pulses in uS
int lcount,rcount;                                            // left and right pulse counters to measure distance
byte encold;                                                  // old state of encoders
byte encnew;                                                  // new state of encoders
byte lstall,rstall;                                           // flags to indicate a motor has stalled
int QEM [16] = {0,1,-1,0,-1,0,0,1,1,0,0,-1,0,-1,1,0};         // Quadrature Encoder Matrix

//============================================================== Motor control variables ========================================================================

int lpwm,rpwm;                                                // left and right motor speeds generated from the processor
int lpow,rpow;
int lspeed;                                                   // left and right motor speeds requested by the user
int rspeed;

unsigned long actual;                                         // stores temporary calculation of actual left and right motor speeds in uS between encoder pulses
unsigned long etime;                                          // timer used to check encoders
unsigned long ltime;                                          // remembers time of left  encoders last state change in uS
unsigned long rtime;                                          // remembers time of right encoders last state change in uS


void setup()
{


  // http://letsmakerobots.com/content/changing-pwm-frequencies-arduino-controllers
  TCCR1B = TCCR1B & B11111000 | B00000101;                    // set timer 1 divisor to  1024 for PWM frequency of    30.64 Hz  best possible torque but vibrates a bit at low speeds.
//TCCR1B = TCCR1B & B11111000 | B00000100;                    // set timer 1 divisor to   256 for PWM frequency of   122.55 Hz  slightly less torque but smoother at low speeds.
//TCCR1B = TCCR1B & B11111000 | B00000011;                    // set timer 1 divisor to    64 for PWM frequency of   490.20 Hz  default value for Arduino - reasonably good torque but a bit noisy.
//TCCR1B = TCCR1B & B11111000 | B00000010;                    // set timer 1 divisor to     8 for PWM frequency of  3921.16 Hz  motors have poor torque and sqeal like a pig in pain!
//TCCR1B = TCCR1B & B11111000 | B00000001;                    // set timer 1 divisor to     1 for PWM frequency of 31372.55 Hz  motors have almost no torque and bats will crash into your windows at night!

  pinMode(lmdirpin,OUTPUT);                                   // configure left  motor DIR pin for output
  pinMode(rmdirpin,OUTPUT);                                   // configure right motor DIR pin for output
  pinMode(lmpwmpin,OUTPUT);                                   // configure left  motor PWM pin for output
  pinMode(rmpwmpin,OUTPUT);                                   // configure right motor PWM pin for output

  Wire.begin(1);                                              // join the I²C bus as slave device at address 1 
  Wire.onReceive(DataReceived);                               // I²C ISR to update motor speeds
  Wire.onRequest(DataSend);                                   // I²C ISR to send distance count
}


void loop()
{ 
  
  
      Serial.print(rcount);
    Serial.println(lcount);
  //============================================================ Encoder Monitor ================================================================================

  while(micros()-etime<100)                                   // wait until 100µS has passed since previous loop                             
  {                                                           // this ensures the encoder states are read every 100µS
  }
  etime=micros();                                             // reset encoder timer so that micros() - etime = 0
  
  encold=encnew;                                              // previous new state now becomes old state
  encnew=PINL&B11110000;                                      // read new encoder states using Arduino Port Manipulation: http://www.arduino.cc/en/Reference/PortManipulation
  
  if((encold&B11000000)!=(encnew&B11000000) || lstall==1)     // compare old state of left encoders with new state of left encoders                  
  {
    lpulse=(micros()-ltime);                                  // time between last state change and this state change
    ltime=micros();                                           // update ltime with time of most recent state change
    lcount+=QEM[(encold&B11000000)*4+(encnew&B11000000)];     // QEM array index is generated using the old and new encoder states. The result determines direction of travel
    actual=25000ul-(abs(lspeed)*90ul);                        // calculate desired time in uS between encoder pulses
    if(actual>lpulse && lpwm>0) lpwm--;                       // if motor is running too fast then decrease PWM
    if(actual<lpulse && lpwm<255) lpwm++;                     // if motor is running too slow then increase PWM
    if(lspeed==0) lpwm=0;                                     // stops imediately instead of ramping down
    analogWrite(lmpwmpin,lpwm);                               // updates left motor power
    digitalWrite(lmdirpin,lspeed<0);                          // updates left motor direction
    lstall=0;                                                 // reset left motor stall flag
  }

  if((encold&B00110000)!=(encnew&B00110000) || rstall==1)     // compare old state of right encoders with new state of right encoders 
  {
    rpulse=(micros()-rtime);                                  // time between last state change and this state change
    rtime=micros();                                           // update ltime with time of most recent state change
    rcount+=QEM[(encold&B00110000)+(encnew&B00110000)/4];     // QEM array index is generated using the old and new encoder states. The result determines direction of travel
    actual=25000ul-(abs(rspeed)*90ul);                        // calculate desired time in uS between encoder pulses
    if(actual>rpulse && rpwm>0) rpwm--;                       // if motor is running too fast then decrease PWM
    if(actual<rpulse && rpwm<255) rpwm++;                     // if motor is running too slow then increase PWM
    if(rspeed==0) rpwm=0;                                     // stops imediately instead of ramping down
    analogWrite(rmpwmpin,rpwm);                               // updates right motor power 
    digitalWrite(rmdirpin,rspeed>0);                          // updates right motor direction
    rstall=0;                                                 // reset right motor stall flag
  }


  //============================================================ Check for stalled motors ============================================
  
  if(micros()-ltime>25000ul)                                  // if left encoder has not changed states within 25mS
  {
    if(lspeed!=0)                                             // if left motor has stalled
    {
      lstall=1;                                               // set left motor stall flag
      lpwm+=2;                                                // bump left motor PWM up a bit
    }
    else
    {
      lpwm=0;                                                 // if motor is supposed to be stopped
      analogWrite(lmpwmpin,lpwm);                             // ensure left motor PWM is 0
    }
  }
  
  if(micros()-rtime>25000ul)                                  // if right encoder has not changed states within 25mS
  {
    if(rspeed!=0)                                             // if right motor has stalled
    {
      rstall=1;                                               // set right motor stall flag
      rpwm+=2;                                                // bump right motor PWM up a bit
    }
    else
    {
      rpwm=0;                                                 // if motor is supposed to be stopped
      analogWrite(rmpwmpin,rpwm);                             // ensure right motor PWM is 0
    }
  }
}






















